
package Spec;

import java.util.*;
import br.ufpe.abaco.compiler.*;

public class SetImplementation {


    CompiledSpecification spec;
    StandardOperators stan;


    SetTerm emptySet;

    SetImplementation(CompiledSpecification s) {
	this.spec = s;
	stan = new StandardOperators(s);

	emptySet = new SetTerm(new HashSet(),stan);

	spec.insertNativeType("set", SetTerm.class, null);

	spec.insertNativeImplementation("set of _", new NativeImplementation() {
		public Term evaluate(Term args[]) {

		    if (args[0].getId() == stan.emptyTupleOp) {
			return emptySet;
		    }

		    HashSet nSet = new HashSet();
		    if (args[0] instanceof TupleTerm) {
			int c;
			TupleTerm tp = (TupleTerm)args[0];
			for (c=tp.size()-1;c>=0;c--)
			    nSet.add(tp.elementAt(c));
		    }
		    else
			nSet.add(args[0]);
		    return new SetTerm(nSet,stan);
		}
	    });
	
	spec.insertNativeImplementation("elements _", new NativeImplementation() {
		public Term evaluate(Term args[]) {
		    if (args[0] instanceof SetTerm) {
			return ((SetTerm)args[0]).elements();
		    }
		    return null;
		}
	    });
	spec.insertNativeImplementation("union _ _", new NativeImplementation() {
		public Term evaluate(Term args[]) {
		    if (args[0] instanceof SetTerm && args[1] instanceof SetTerm) {
			return ((SetTerm)args[0]).union((SetTerm)args[1]);
		    }
		    return null;
		}
	    });

	spec.insertNativeImplementation("difference ( _ , _ )", new NativeImplementation() {
		public Term evaluate(Term args[]) {
		    if (args[0] instanceof SetTerm && args[1] instanceof SetTerm) {
			return ((SetTerm)args[0]).difference((SetTerm)args[1]);
		    }
		    return null;
		}
	    });

	spec.insertNativeImplementation("intersection _ _", new NativeImplementation() {
		public Term evaluate(Term args[]) {
		    if (args[0] instanceof SetTerm && args[1] instanceof SetTerm) {
			return ((SetTerm)args[0]).intersection((SetTerm)args[1]);
		    }
		    return null;
		}
	    });
	
	
	spec.insertNativeImplementation("_ is in _", new NativeImplementation() {
		public Term evaluate(Term args[]) {
		    if (args[1] instanceof SetTerm) {
			if (((SetTerm)args[1]).isIn(args[0]))
			    return getTrue();
			else
			    return getFalse();
		    }
		    return null;
		}
	    });
	
	spec.insertNativeImplementation("_ is included in _", new NativeImplementation() {
		public Term evaluate(Term args[]) {
		    if (args[0] instanceof SetTerm && args[1] instanceof SetTerm) {
			if (((SetTerm)args[1]).isIncluded((SetTerm)args[0]))
			    return getTrue();
			else
			    return getFalse();
		    }
		    return null;
		}
	    });

    }

    Term vTrue = null;
    Term getTrue() {
	if (vTrue == null) {
	    try { 
		vTrue = spec.getOperator("true").apply(new Term[0]);
	    } catch (NothingException ne) { }
	}
	return vTrue;
    }
    Term vFalse = null;
    Term getFalse() {
	if (vFalse == null) {
	    try { 
		vFalse = spec.getOperator("false").apply(new Term[0]);
	    } catch (NothingException ne) { }
	}
	return vFalse;
    }
    
    public static void register(CompiledSpecification spec) {
	new SetImplementation(spec);
    }

}
